
class Recipe:
    def __init__(self):
        self.name = "Unknown"
        self.author = "Unknown"
        self.description = ""
        self.yeild = 4
        self.category = "Unknown"
        self.ingredients = []
        self.instructions = []
        self.prep_time = "PT0M"
        self.cook_time = "PT0M"
        self.image = ""

    
